/*ywc 2005-01-21 add for flash simulation*/
//core flash simulation source code from ipaqsim
//Thanks to ipaqsim's AUTHOR(s):     Ye Wen (wenye@cs.ucsb.edu) 

#include <stdio.h>
#include "armdefs.h"
#include "skyeye_flash.h"
#include "armmem.h"

/*ywc 2005-04-01*/
void init_querytable ();

ARMword *mem;			/* data */
ARMWord *mem;			/* data */
char *lock;			/* sector lock */
unsigned long size;		/* byte size */
	//ARMAddr base;                 /* start of flash bank */

ARMWord read_mode;		/* mode for read operation */
ARMWord wsm_mode;		/* write state machine */

static ARMWord query[INTEL_QUERYTABLE_SIZE];	/* query table */

	/*
	 * state bits for handlers: program, buffer program, erase, lock 
	 * xxxx_busy: whether WSM is busy on the handler
	 * xxxx_suspended: whether the handler is suspended
	 * xxxx_error: operation errors
	 */
ARMByte program_busy, progbuf_busy, erase_busy, lock_busy;
ARMByte program_suspended, progbuf_suspended, erase_suspended;
ARMByte protection_error, program_setlb_error, erase_clearlb_error,
  program_volt_error;

	/* latch address&data for each handlers */
ARMAddr program_latch_addr, program_latch_data;
ARMAddr progbuf_latch_addr, progbuf_latch_data;
ARMAddr erase_latch_addr, erase_latch_data;

	/* 
	 * stateful information for buffer program handler 
	 */
	/* total count and number to be loaded/programmed */
ARMWord pb_count, pb_loaded;
ARMAddr pb_start;		/* start address */
ARMWord pb_buf[INTEL_WRITEBUFFER_SIZE];	/* write buffer */

	/* VPEN pin: should controlled by EGPIO ??? */
ARMByte vpen;

/* return the lock bit */
#define ISLOCKED(x)                   (lock[(x)>>FLASH_SECTOR_SHIFT])


#define ADDR_SUSPENDED(x)       \
((program_suspended && ((x) == program_latch_addr)) || \
 (progbuf_suspended && (((x) >= pb_start) && ((x) < (pb_start+pb_count)))) || \
 (erase_suspended && (((x) & FLASH_SECTOR_MASK) == (erase_latch_addr & FLASH_SECTOR_MASK))) )

#define DEVICE_SUSPENDED        (program_suspended || progbuf_suspended || erase_suspended)

void
Intel28F128J3A_flash_init (ARMul_State * state)
{
  unsigned int i;

  size = INTEL28F128J3A_SIZE;
  //base = 0x00000000;
  /*
     if (this->mem == NULL){
     printf("\nFlash memory allocation failed");          // exit 
     }
   */
  /*
     for (unsigned int i = 0; i < size >> WORD_SHIFT; i ++){
     //this->mem[i] = 0xffffffff; // initial state of flash
     }
   */

  lock = malloc (size / FLASH_SECTOR_SIZE);
  if (lock == NULL)
    {
      printf ("\nflash memory lock allocation failed");	/* exit */
    }
  for (i = 0; i < size / FLASH_SECTOR_SIZE; i++)
    {
      lock[i] = 0;
    }
  read_mode = WSM_READ_ARRAY;
  wsm_mode = WSM_READY;

  program_busy = progbuf_busy = erase_busy = lock_busy = 0;
  program_suspended = progbuf_suspended = erase_suspended = 0;
  protection_error = program_setlb_error = erase_clearlb_error =
    program_volt_error = 0;

  //vpen = 0;     /* disable program/erase */
  vpen = 1;			/* enable program/erase */

  pb_count = pb_loaded = 0;

  init_querytable ();
}

ARMword
Intel28F128J3A_flash_read_byte (ARMul_State * state, ARMword addr)
{
  ARMword data, offset;
  data = Intel28F128J3A_flash_read_word (state, addr);
  offset = (((ARMword) state->bigendSig * 3) ^ (addr & 3)) << 3;
  return (data >> offset & 0xffL);
}

ARMword
Intel28F128J3A_flash_read_halfword (ARMul_State * state, ARMword addr)
{
  ARMword data, offset;
  data = Intel28F128J3A_flash_read_word (state, addr);
  offset = (((ARMword) state->bigendSig * 2) ^ (addr & 2)) << 3;
  return (data >> offset & 0xffffL);
}

ARMword
Intel28F128J3A_flash_read_word (ARMul_State * state, ARMword addr)
{

  ARMword temp;

  ARMword data;

  switch (read_mode)
    {
    case WSM_READ_ARRAY:	/* read flash */
      //data = mem[WORD_ADDR(addr)];
      data = real_read_word (state, addr);
      //data = state->mem.rom[mbp - skyeye_config.mem.mem_banks][(addr- mbp->addr)>>2];
      break;
    case WSM_READ_ID:		//read IDs
      temp = WORD_ADDR (addr) & 0x00000001;
      if (temp == 0)
	{
	  data = BOTHCHIP (INTEL_MANUFACTURER_CODE);
	}			//manu. ID 
      else if (temp == 1)
	{
	  data = BOTHCHIP (INTEL_28F128J3A_DEVICE_CODE);
	}			// device ID
      else if (((addr & FLASH_SECTOR_OFF) >> WORD_SHIFT) == 2)	//read lock state 
	data = BOTHCHIP (ISLOCKED (addr) & 0x1);
      else
	{
	  data = 0;
	  printf ("\nFlash: read ID error: unknown address 0x%x\n", addr);
	}
      break;
    case WSM_READ_STATUS:	/* read status register */

      /* first get the WSM busy/ready state */
      temp = !program_busy && !progbuf_busy && !lock_busy && !erase_busy;
      /* suppose no voltage error */
      data = BOTHCHIP ((temp << 7) |
		       (erase_suspended << 6) |
		       (erase_clearlb_error << 5) |
		       (program_setlb_error << 4) |
		       (program_volt_error << 3) |
		       (program_suspended << 2) | (protection_error) << 1);

      break;
    case WSM_READ_QUERY:	// read query table 
      //temp = WORD_ADDR(addr);
      temp = WORD_ADDR (addr & 0x0000ffff);
      data = (temp < INTEL_QUERYTABLE_SIZE) ? query[temp] : 0;
      break;
    default:
      printf ("\nFlash: invalid read mode: %d", read_mode);
      data = 0;
      break;
    }
  return data;
}

void
Intel28F128J3A_flash_write_byte (ARMul_State * state, ARMword addr,
				 ARMword data)
{
  printf ("\nFlash in current config does not support halfword write at 0x%x",
	  addr);
  //Intel28F128J3A_flash_write_word(state,addr,data);
}

void
Intel28F128J3A_flash_write_halfword (ARMul_State * state, ARMword addr,
				     ARMword data)
{
  printf ("\nFlash in current config does not support byte write at 0x%x",
	  addr);
  //Intel28F128J3A_flash_write_word(state,addr,data);
}

/* initialize a new WSM command sequence */
void
init_wsm (ARMAddr addr, ARMWord data)
{
  /* maybe we don't need to care the data symmetry */
  //CHECK_DATA_VALID(data);

  /*
   * No timing support now, so Suspend/Resume has no effect 
   * July 1st, 2004 Ye Wen
   */
  int i;
  ARMWord cmd = data & 0xff;
  switch (cmd)
    {
    case WSM_READ_ARRAY:
      read_mode = WSM_READ_ARRAY;
      break;
    case WSM_READ_ID:
      read_mode = WSM_READ_ID;
      break;
    case WSM_READ_STATUS:
      read_mode = WSM_READ_STATUS;
      break;
    case WSM_READ_QUERY:
      read_mode = WSM_READ_QUERY;
      break;
    case WSM_CLEAR_STATUS:
      protection_error = program_setlb_error =
	erase_clearlb_error = program_volt_error = 0;
      // is this right? I can't see it from spec.
      // but the bootldr code implies this. Check
      // program_flash_region in bootldr.c
      // -July 28, 2004 Ye Wen
      read_mode = WSM_READ_ARRAY;
      break;
    case WSM_PROGRAM:
    case WSM_PROGRAM2:
      wsm_mode = WSM_PROGRAM;
      break;
    case WSM_WRITE_BUFFER:
      wsm_mode = WSM_WRITE_BUFFER;
      read_mode = WSM_READ_STATUS;
      pb_count = pb_loaded = 0;
      for (i = 0; i < INTEL_WRITEBUFFER_SIZE; i++)
	pb_buf[i] = 0xffffffff;
      break;
    case WSM_BLOCK_ERASE:
      wsm_mode = WSM_BLOCK_ERASE;
      break;
    case WSM_SUSPEND:
      if (program_busy)
	program_suspended = 1;
      else if (progbuf_busy)
	progbuf_suspended = 1;
      else if (erase_busy)
	erase_suspended = 1;
      else
	{
	  //ywc
	  //printf("\nFlash: nothing busy for suspending");
	}
      break;
    case WSM_RESUME:
      if (program_suspended)
	program_suspended = 0;
      else if (progbuf_suspended)
	progbuf_suspended = 0;
      else if (erase_suspended)
	erase_suspended = 0;
      else
	{
	  //ywc
	  //printf("\nFlash: nothing to resume");
	}
      break;
    case WSM_LOCK_ACCESS:
      wsm_mode = WSM_LOCK_ACCESS;
      break;
    case WSM_CONFIG:
    case WSM_PROTECT:
      printf ("\nFlash: command 0x%x not supported yet", cmd);
      break;
    default:
      //ywc
      //printf("\nFlash: command 0x%x unrecognized", cmd);
      break;
    }
}
void
Intel28F128J3A_flash_write_word (ARMul_State * state, ARMword addr,
				 ARMword data)
{
  unsigned int i;
  ARMAddr j;

  //CHECK_ADDR_RANGE(addr);

  switch (wsm_mode)
    {
    case WSM_READY:
      init_wsm (addr, data);
      break;
    case WSM_WRITE_BUFFER:
      if (pb_count == 0)
	{			/* step 1: get count */
	  pb_count = (data & 0xff) + 1;
	  progbuf_latch_addr = addr;
	  if (pb_count > INTEL_WRITEBUFFER_SIZE)
	    {
	      program_setlb_error = erase_clearlb_error = 1;
	      wsm_mode = WSM_READY;
	      printf ("\nFlash: buffer program count too large: %d",
		      pb_count);
	      break;
	    }
	  if (SECTOR_ADDR (addr) != SECTOR_ADDR (addr + pb_count - 1))
	    {
	      program_setlb_error = erase_clearlb_error = 1;
	      wsm_mode = WSM_READY;
	      printf
		("\nFlash: buffer program address range across sectors: 0x%x-0x%x",
		 addr, addr + pb_count - 1);
	      break;
	    }
	  break;
	}
      if (pb_loaded < pb_count)
	{			/* step 2: load data */
	  if (SECTOR_ADDR (addr) != SECTOR_ADDR (progbuf_latch_addr))
	    {
	      program_setlb_error = erase_clearlb_error = 1;
	      wsm_mode = WSM_READY;
	      printf ("\nFlash: buffer program address across sectors: 0x%x",
		      addr);
	      break;
	    }
	  if (pb_loaded == 0)
	    pb_start = addr;
	  pb_buf[WORD_ADDR (addr) & INTEL_WRITEBUFFER_MASK] = data;
	  pb_loaded++;
	  break;
	}
      if ((data & 0xff) == WSM_CONFIRM)
	{			/* step 3: confirm the program */
	  progbuf_busy = 1;
	  read_mode = WSM_READ_STATUS;
	  /* copy buffer to flash */
	  if (ISLOCKED (progbuf_latch_addr))
	    {
	      printf ("\nFlash: buffer program locked address [0x%x]",
		      progbuf_latch_addr);
	      program_setlb_error = protection_error = 1;
	    }
	  else if (DEVICE_SUSPENDED)
	    {
	      printf ("\nFlash: buffer program suspended device");
	      program_setlb_error = 1;
	    }
	  else if (vpen != 1)
	    {
	      printf ("\nFlash: can't buffer program when VPEN low");
	      program_setlb_error = program_volt_error = 1;
	    }
	  else
	    {
	      for (j = WORD_ADDR (pb_start);
		   j < WORD_ADDR (pb_start) + INTEL_WRITEBUFFER_SIZE; j++)
		//mem[j] &= pb_buf[j & INTEL_WRITEBUFFER_MASK];
		real_write_word (state, j,
				 real_read_word (state,
						 j) & pb_buf[j &
							     INTEL_WRITEBUFFER_MASK]);
	    }
	  wsm_mode = WSM_READY;
	  progbuf_busy = 0;
	  break;
	}
      else
	{
	  printf ("\nFlash: buffer program with wrong cmd sequence: 0x%x",
		  data);
	  program_setlb_error = erase_clearlb_error = 1;
	  wsm_mode = WSM_READY;
	  break;
	}
      break;
    case WSM_PROGRAM:
    case WSM_PROGRAM2:
      program_busy = 1;
      program_latch_addr = addr;
      program_latch_data = data;
      read_mode = WSM_READ_STATUS;
      if (ISLOCKED (program_latch_addr))
	{
	  printf ("\nFlash: program locked address [0x%x]=0x%x", addr, data);
	  program_setlb_error = protection_error = 1;
	}
      else if (ADDR_SUSPENDED (program_latch_addr))
	{
	  printf ("\nFlash: program suspended address [0x%x]=0x%x", addr,
		  data);
	  program_setlb_error = 1;
	}
      else if (vpen != 1)
	{
	  printf ("\nFlash: can't program when VPEN low");
	  program_setlb_error = program_volt_error = 1;
	}
      else
	{
	  //mem[WORD_ADDR(program_latch_addr)] &= data;
	  real_write_word (state, program_latch_addr,
			   real_read_word (state, program_latch_addr) & data);
	}
      program_busy = 0;
      wsm_mode = WSM_READY;
      break;
    case WSM_BLOCK_ERASE:
      if ((data & 0xff) == WSM_CONFIRM)
	{
	  erase_busy = 1;
	  erase_latch_addr = addr & FLASH_SECTOR_MASK;
	  read_mode = WSM_READ_STATUS;
	  if (ISLOCKED (erase_latch_addr))
	    {
	      printf ("\nFlash: erase locked address [0x%x]",
		      erase_latch_addr);
	      erase_clearlb_error = protection_error = 1;
	    }
	  else if (DEVICE_SUSPENDED)
	    {
	      printf ("\nFlash: erase suspended device");
	      erase_clearlb_error = 1;
	    }
	  else if (vpen != 1)
	    {
	      printf ("\nFlash: can't erase when VPEN low\n");
	      erase_clearlb_error = program_volt_error = 1;
	    }
	  else
	    {
	      for (i = 0; i < WORD_ADDR (FLASH_SECTOR_SIZE); i++)
		{
		  //mem[WORD_ADDR(erase_latch_addr)+i] = 0xffffffff;
		  real_write_word (state, erase_latch_addr + i * 4,
				   0xffffffff);
		}
	    }
	  erase_busy = 0;
	  wsm_mode = WSM_READY;
	  break;
	}
      else
	{
	  printf ("\nFlash: erase with wrong cmd sequence: 0x%x", data);
	  wsm_mode = WSM_READY;
	  break;
	}
      break;
    case WSM_LOCK_ACCESS:
      if ((data & 0xff) == 0x01)
	{			/* set lock bit */
	  lock_busy = 1;
	  read_mode = WSM_READ_STATUS;
	  if (DEVICE_SUSPENDED)
	    {
	      printf ("\nFlash: set lock on suspended device");
	      program_setlb_error = 1;
	    }
	  else if (vpen != 1)
	    {
	      printf ("\nFlash: can't set lock when VPEN low");
	      program_setlb_error = program_volt_error = 1;
	    }
	  else
	    {
	      lock[SECTOR_ADDR (addr)] = 1;
	    }
	  lock_busy = 0;
	  wsm_mode = WSM_READY;
	  break;
	}
      else if ((data & 0xff) == WSM_CONFIRM)
	{			/* clear all lock bits */
	  lock_busy = 1;
	  read_mode = WSM_READ_STATUS;
	  if (DEVICE_SUSPENDED)
	    {
	      printf ("\nFlash: clear locks on suspended device");
	      erase_clearlb_error = 1;
	    }
	  else if (vpen != 1)
	    {
	      printf ("\nFlash: can't clear locks when VPEN low");
	      erase_clearlb_error = program_volt_error = 1;
	    }
	  else
	    {
	      for (i = 0; i < size / FLASH_SECTOR_SIZE; i++)
		lock[i] = 0;
	    }
	  lock_busy = 0;
	  wsm_mode = WSM_READY;
	  break;
	}
      else
	{
	  printf ("\nFlash: lock access with wrong cmd sequence: 0x%x", data);
	  wsm_mode = WSM_READY;
	  break;
	}
      break;
    default:
      printf ("\nFlash: Can't recognize WSM mode: 0x%x", wsm_mode);
      break;
    }

}

void
init_querytable ()
{
  int i;
  for (i = 0; i < INTEL_QUERYTABLE_SIZE; i++)
    {
      query[i] = 0;
    }

  /*
   * This is a dump of a real 28F128J3A flash query table 
   * Lines with a "*": different from Verilog model (Ref.2)
   * Lines with a "+": different from the spec (Ref.1)
   */
  query[0x00] = 0x00890089;	//manu. ID
  //query[0x01] = 0x00170017;             //device ID     
  query[0x01] = 0x00180018;	//device ID     
  query[0x02] = 0x00010001;	//?

  query[0x10] = 0x00510051;	//"Q"
  query[0x11] = 0x00520052;	//"R"
  query[0x12] = 0x00590059;	//"Y"

  query[0x13] = 0x00010001;

  query[0x15] = 0x00310031;

  query[0x1b] = 0x00270027;
  query[0x1c] = 0x00360036;
  query[0x1f] = 0x00070007;
  query[0x20] = 0x00070007;
  query[0x21] = 0x000A000A;
  query[0x23] = 0x00040004;
  query[0x24] = 0x00040004;
  query[0x25] = 0x00040004;

  query[0x27] = 0x00180018;	//2^(0x18) = 2^24 = 16MByte 28F128 flash chip size
  //query[0x27] = 0x00170017;

  query[0x28] = 0x00020002;
  query[0x2a] = 0x00050005;	//2^5 = 32 write buffer size
  query[0x2c] = 0x00010001;	//symmetrically-blocked

  query[0x2d] = 0x007F007F;	//[31,16]=?(128KByte) ;[15,0]=block number

  query[0x30] = 0x00020002;

  query[0x31] = 0x00500050;	//"P"
  query[0x32] = 0x00520052;	//"R"
  query[0x33] = 0x00490049;	//"I"

  query[0x34] = 0x00310031;
  query[0x35] = 0x00320032;	// *

  query[0x36] = 0x00CE00CE;	// *

  query[0x3a] = 0x00010001;

  query[0x3b] = 0x00010001;
  query[0x3c] = 0x00010001;

  query[0x3d] = 0x00330033;
  query[0x3f] = 0x00010001;

  query[0x40] = 0x00000000;	// +
  query[0x41] = 0x00010001;	// +
  query[0x42] = 0x00030003;
  query[0x43] = 0x00030003;
  query[0x44] = 0x00030003;
}

/*
// ywc 2005-02-02
//inline void set_vpen() { vpen = 1; }
//inline void clear_vpen() { vpen = 0; }
void Intel28F128J3A_flash_set_vpen(ARMul_State *state){
	vpen = 1; 
}

void Intel28F128J3A_flash_clear_vpen(ARMul_State *state){
	vpen = 0; 
}
*/
